#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#define ITERS 300
#define BIG	1000
#define MAXACOLOR 2048
#define MAXBCOLOR 255

typedef struct {
/*Complex number definition*/
	double re, im;
} complex;

complex csum (complex a, complex b){
/*Complex sum*/
	complex c;
	c.re = a.re + b.re;
	c.im = a.im + b.im;
	return c;
}

complex cquad (complex a){
/*Complex square*/
	complex c;
	c.re = a.re*a.re - a.im*a.im;
	c.im = 2 * a.re * a.im;
	return c;
}

double cmod (complex a){
/*Complex module*/
	return 	sqrt(a.re*a.re + a.im*a.im);
}

complex sucession (complex x, complex c){
/*Mandelbrot set*/
	return csum(cquad(x),c);
}

void rdata(char *myfile, double *x){
/* This function reads the data that provides the range
 * to construct the complex numbers with format
 * 
 * Imaginary_part_(minimum) Imaginary_part_(maximum) Imaginary_part_(range_width)
 * Real_part_(minimum) Real_part_(maximum) Real_part_(range_width)
 *
 * Values are stored in array x
 * */
	FILE *f = fopen(myfile,"r");
		fscanf(f, "%lf %lf %lf\n", &x[0], &x[1], &x[2]);
		fscanf(f, "%lf %lf %lf", &x[3], &x[4], &x[5]);
	fclose(f);
	return;
}

int iterate(complex c, complex (*f)(complex, complex)){
/* This function iterates the f function ITERS times or while its value 
 * is lower than BIG. 
 * 
 * It returns the number of iterations done.
 * */
	complex x;
	int i = 0;
	x.re = 0;
	x.im = 0;
	while (i<ITERS && cmod(x)<BIG){
		x = f(x,c);
		i++;
	}
	return i;
}

typedef struct{
/*Ascii RGB color definition*/
	char r,g,b;
	} rgbcolor;


typedef struct{
/*Integer RGB color definition*/
	int r,g,b;
	} irgbcolor;



rgbcolor getcolor (int c){
/* This function takes an integer number between 0 and ITERS and
 * returns a RGB color (typedef rgbcolor) */
		rgbcolor thiscolor;
		if (c==ITERS){
			thiscolor.r = MAXBCOLOR;
			thiscolor.g = MAXBCOLOR;
			thiscolor.b = MAXBCOLOR;
		}else{
			thiscolor.r = (char) (MAXBCOLOR - c*MAXBCOLOR/ITERS);
			thiscolor.g = 0;
			thiscolor.b = (char) (c*MAXBCOLOR/ITERS);			
		}
		return thiscolor;
	}


irgbcolor geticolor (int c){
/* This function takes an integer number between 0 and ITERS and
 * returns a RGB color (typedef irgbcolor) */

		irgbcolor thiscolor;
		if (c==ITERS){
			thiscolor.r = MAXACOLOR;
			thiscolor.g = 0;
			thiscolor.b = 0;
		}else{
			thiscolor.r = 0;
			thiscolor.g = (int) (c*MAXACOLOR/ITERS);
			thiscolor.b = (int) (MAXACOLOR - c*MAXACOLOR/ITERS);
		}
		return thiscolor;
	}


int makeppm(char *myfile, int *p, int nf, int nc){
/* This function makes a PPM image from the data stored in the 
 * pointer array p with nf (rows) y nc (colunms) dimenisions */	
	int i, j;
	char col;
	FILE *of;
	rgbcolor mycolor;
	of = fopen(myfile,"w");
	fprintf(of, "P6\n# Mandelbrot Set\n%d %d\n%d\n", nc, nf, MAXBCOLOR);
	for (i=0;i<nf;i++){
		for (j=0;j<nc;j++){
			mycolor = getcolor(*p);
			fprintf(of,"%c%c%c", mycolor.r, mycolor.g, mycolor.b);
			p++;
		}
	}
	fclose(of);	
	return 0;
}

int makeppm_ascii(char *myfile, int *p, int nf, int nc){
/* This function makes an ascii  PGM image from the data stored in the 
 * pointer array p with nf (rows) y nc (colunms) dimenisions */	
 	int i, j;
	irgbcolor mycolor;
	FILE *of;
	of = fopen(myfile,"w");
	fprintf(of, "P3\n# Mandelbrot Set\n%d %d\n%d\n", nc, nf, MAXACOLOR);
	for (i=0;i<nf;i++){
		for (j=0;j<nc;j++){
			mycolor = geticolor(*p);
			fprintf(of,"%d %d %d ", mycolor.r, mycolor.g, mycolor.b);
			p++;
		}
		fprintf(of, "\n");
	}
	fclose(of);	
	return 0;
}


int makepgm(char *myfile, int *p, int nf, int nc){
/* This function makes an integer PGM image from the data stored in the 
 * pointer array p with nf (rows) y nc (colunms) dimenisions */	
 	int i, j;
	FILE *of;
	of = fopen(myfile,"w");
	fprintf(of, "P2\n# Mandelbrot Set\n%d %d\n%d\n", nc, nf, MAXACOLOR);
	for (i=0;i<nf;i++){
		for (j=0;j<nc;j++){
			fprintf(of,"%d ", ((*p)==ITERS)?MAXACOLOR:(int) (MAXACOLOR - (*p)*MAXACOLOR/ITERS));
			p++;
		}
		fprintf(of, "\n");
	}
	fclose(of);	
	return 0;
}


int main(int argc, char *argv[]){
	int i, j, nf, nc;
	complex x,c;
	double cc[6];
	char *myfile;
	int *p;
	myfile = argv[1]; //reads the name of input data file from command line
	rdata(myfile, cc); // reads the input data
	nf = ((cc[1]-cc[0])/cc[2])+1; //gets the number of rows for the matrix
	nc = ((cc[4]-cc[3])/cc[5])+1; //gets the number of colunms for the matrix
	p = (int *) malloc(nf*nc*sizeof(int)); //Dimesions the matrix
	for (i=0;i<nf;i++){
		c.im = cc[0]+i*cc[2];
		for (j=0;j<nc;j++){
			c.re = cc[3]+j*cc[5];
			*p = iterate(c,&sucession); //Iterates the sucession for every value of c
			p++;
		}
	}
	p-=nf*nc; //goes to the matrix first pointer
	makeppm_ascii("mandelbrot.ppm", p, nf, nc); //paints the color image
	makepgm("mandelbrot.pgm", p, nf, nc); //paints the grayscale image
	return 0;
}
